@@ -2,7 +2,7 @@
 sections for nox tests, which include tags that indicate various combinations
 of those parameters in such a way that it's somewhat similar to how
 we were using the tox project; where individual dash-separated tags could
-be added to add more specificity to the suite configuation, or omitting them
+be added to add more specificity to the suite configuration, or omitting them
 would fall back to defaults.
 
 
@@ -11,6 +11,7 @@ would fall back to defaults.
 from __future__ import annotations
 
 import collections
+import os
 import re
 import sys
 from typing import Any
@@ -204,3 +205,48 @@ def extract_opts(posargs: list[str], *args: str) -> tuple[list[str], Any]:
     return [arg for arg in posargs if not extract(arg)], return_tuple(
         *return_args
     )
+
+
+def apply_pytest_opts(
+    session: nox.Session,
+    cov: str,
+    tokens: list[str],
+    *,
+    coverage: bool = False,
+) -> list[str]:
+    posargs, opts = extract_opts(session.posargs, "generate-junit")
+
+    if session.python and isinstance(session.python, str):
+        python_token = session.python.replace(".", "")
+        tokens.insert(0, python_token)
+
+    file_suffix = "-".join(t for t in tokens if not t.startswith("_"))
+
+    if coverage:
+
+        session.env["COVERAGE_FILE"] = coverage_file = (
+            f".coverage.{file_suffix}"
+        )
+        coverage_xml_file = f"coverage-{file_suffix}.xml"
+
+        if os.path.exists(coverage_file):
+            os.unlink(coverage_file)
+        posargs.extend(
+            [
+                f"--cov={cov}",
+                "--cov-append",
+                "--cov-report",
+                "term",
+                "--cov-report",
+                f"xml:{coverage_xml_file}",
+            ],
+        )
+        session.log(f"Will store coverage data in {coverage_file}")
+        session.log(f"Will write xml coverage data to {coverage_xml_file}")
+
+    if opts.generate_junit:
+        junitfile = f"junit-{file_suffix}.xml"
+        session.log(f"Will store junit xml in {junitfile}")
+        posargs.extend(["--junitxml", junitfile])
+
+    return posargs
